@@ -13,7 +13,7 @@ import yaml
 from goofi.connection import Connection
 from goofi.gui.window import Window
 from goofi.message import Message, MessageType
-from goofi.node import MultiprocessingForbiddenError
+from goofi.node import MultiprocessingForbiddenError, Node
 from goofi.node_helpers import NodeProcessRegistry, NodeRef, list_nodes
 
 
@@ -83,7 +83,7 @@ class NodeContainer:
         if name in self._nodes:
             self._nodes[name].terminate()
             del self._nodes[name]
-            return True
+            return
         raise KeyError(f"Node {name} not in container")
 
     def __getitem__(self, name: str) -> NodeRef:
@@ -157,6 +157,9 @@ class Manager:
         self._running = True
         self.nodes = NodeContainer()
 
+        # propagate the headless argument to the node process registry
+        NodeProcessRegistry().headless = headless
+
         # store attributes related to loading and saving
         self._save_path = None
         self._unsaved_changes = False
@@ -304,20 +307,23 @@ class Manager:
 
         # import the node
         mod = importlib.import_module(f"goofi.nodes.{category}.{node_type.lower()}")
-        node = getattr(mod, node_type)
+        node: Node = getattr(mod, node_type)
 
         # instantiate the node
         ref = None
         if self._use_multiprocessing:
             # try to spawn the node in a separate process
             try:
-                ref = node.create(initial_params=params)
+                ref = node.create(initial_params=params, send_output_to_ref=not self.headless)
             except MultiprocessingForbiddenError:
                 # the node doesn't support multiprocessing, create it in the local process
                 pass
         if ref is None:
             # spawn the node in the local process
-            ref = node.create_local(initial_params=params)[0]
+            ref = node.create_local(initial_params=params, send_output_to_ref=not self.headless)[0]
+
+        # set up the shutdown callback handler to terminate the manager
+        ref.set_message_handler(MessageType.SHUTDOWN, lambda *args: self.terminate())
 
         # add the node to the container
         if name is None:
@@ -373,6 +379,10 @@ class Manager:
         `gui_kwargs` : dict
             Additional keyword arguments to pass to the gui.
         """
+        # TODO: use a thread connection that avoids serialization in case the two nodes share a process
+        # if self.nodes[node_in].process == self.nodes[node_out].process:
+        #     use a thread connection
+
         # TODO: Prevent multiple links to the same input slot. The GUI already prevents this, but the manager should too.
         self.nodes[node_out].connection.send(
             Message(
@@ -635,27 +645,38 @@ def get_example_patch(args) -> bool:
         return True
 
 
-def main(duration: float = 0, args=None):
+def main(duration: Optional[float] = None, args=None):
     """
     This is the main entry point for goofi-pipe. It parses command line arguments, creates a manager
     instance and runs all nodes until the manager is terminated.
 
     ### Parameters
     `duration` : float
-        The duration to run the manager for. If `0`, runs indefinitely.
+        The duration to run the manager for. Runs indefinitely if None or `0`.
     `args` : list
         A list of arguments to pass to the manager. If `None`, uses `sys.argv[1:]`.
     """
     import argparse
 
-    comm_choices = list(Connection.get_backends().keys())
+    comm_choices = list(Connection.get_ipc_backends().keys())
 
     # parse arguments
     parser = argparse.ArgumentParser(description="goofi-pipe")
     parser.add_argument("filepath", nargs="?", help="path to the file to load from")
     parser.add_argument("--headless", action="store_true", help="run in headless mode")
     parser.add_argument("--no-multiprocessing", action="store_true", help="disable multiprocessing")
-    parser.add_argument("--comm", choices=["auto"] + comm_choices, default="auto", help="node communication backend")
+    parser.add_argument(
+        "--ipc-backend",
+        choices=["auto"] + comm_choices,
+        default="auto",
+        help="node inter-process communication backend",
+    )
+    parser.add_argument(
+        "--duration",
+        default=0,
+        type=float,
+        help="Duration (in seconds) after which goofi-pipe automatically shuts down (0 to run indefinitely)",
+    )
     parser.add_argument("--update-readme-docs", action="store_true", help="update the node list in the README")
     parser.add_argument(
         "--gen-node-docs", action="store_true", help="generate missing node docstrings using the openai API"
@@ -682,13 +703,20 @@ def main(duration: float = 0, args=None):
         if not get_example_patch(args):
             return
 
+    if duration is not None and args.duration != 0:
+        raise ValueError(
+            "Manager duration should be given either as a parameter or as a command line argument, not both."
+        )
+    elif duration is None:
+        duration = args.duration
+
     # create and run the manager (this blocks until the manager is terminated)
     Manager(
         filepath=args.filepath,
         headless=args.headless,
         use_multiprocessing=not args.no_multiprocessing,
         duration=duration,
-        communication_backend=args.comm,
+        communication_backend=args.ipc_backend,
     )
 
 
