@@ -6,7 +6,7 @@ from copy import deepcopy
 from enum import Enum
 from multiprocessing import Process
 from os.path import dirname, join
-from pathlib import PosixPath
+from pathlib import Path
 from threading import Event, Thread
 from typing import Any, Callable, Dict, Optional, Tuple, Union
 
@@ -98,8 +98,9 @@ class Node(ABC):
         self._input_slots = input_slots
         self._output_slots = output_slots
         self._params = params
+        self._environment = environment
 
-        self._validate_attrs(environment)
+        self._validate_attrs()
 
         # NOTE: we avoid creating a threading.Event() in standalone mode to ensure nodes can be pickled
         if environment != NodeEnv.STANDALONE:
@@ -139,12 +140,12 @@ class Node(ABC):
             raise ValueError(f"Invalid environment: {environment}")
 
     @require_init
-    def _validate_attrs(self, environment: NodeEnv) -> None:
+    def _validate_attrs(self) -> None:
         """
         Check that all attributes are present and of the correct type.
         """
         # check connection type
-        if environment == NodeEnv.STANDALONE:
+        if self._environment == NodeEnv.STANDALONE:
             if self.connection is not None:
                 raise ValueError("Running in standalone mode, connection should be None.")
         else:
@@ -474,7 +475,7 @@ class Node(ABC):
                     t.start()
                     self.pending_connections[conn._id].append((t, time.time()))
 
-    @staticmethod
+    @classmethod
     def _configure(cls) -> Tuple[Dict[str, InputSlot], Dict[str, OutputSlot], NodeParams]:
         """Retrieves the node's configuration of input slots, output slots, and parameters."""
         in_slots = cls.config_input_slots()
@@ -497,8 +498,22 @@ class Node(ABC):
         """Clear the error message."""
         self.connection.try_send(Message(MessageType.PROCESSING_ERROR, {"error": None}))
 
+    @require_init
+    def request_shutdown(self):
+        """Send a shutdown request for the goofi-pipe process."""
+        if self._environment == NodeEnv.STANDALONE:
+            raise RuntimeError(
+                "Shutdown requested but the node is not running in standalone mode. A manager is required to perform the shutdown."
+            )
+        self.connection.try_send(Message(MessageType.SHUTDOWN, {}))
+
     @classmethod
-    def create(cls, initial_params: Optional[Dict[str, Dict[str, Any]]] = None, retries: int = 3) -> NodeRef:
+    def create(
+        cls,
+        initial_params: Optional[Dict[str, Dict[str, Any]]] = None,
+        retries: int = 3,
+        send_output_to_ref: bool = True,
+    ) -> NodeRef:
         """
         Create a new node instance in a separate process and return a reference to the node.
 
@@ -506,6 +521,10 @@ class Node(ABC):
         `initial_params` : Optional[Dict[str, Dict[str, Any]]]
             A dictionary of parameter groups, where each group is a dictionary of parameter names and values.
             Defaults to None.
+        `retries` : int
+            The number of times to retry creating the node if it fails. Defaults to 3.
+        `send_output_to_ref` : bool
+            If True, connect the output slots of the node to the returned NodeRef. Defaults to True.
 
         ### Returns
         `NodeRef`
@@ -526,7 +545,7 @@ class Node(ABC):
                 return ref
 
         # generate arguments for the node
-        in_slots, out_slots, params = cls._configure(cls)
+        in_slots, out_slots, params = cls._configure()
         # integrate initial parameters if they are provided
         if initial_params is not None:
             try:
@@ -567,6 +586,7 @@ class Node(ABC):
             params,
             cls,
             process=proc,
+            receive_node_outputs=send_output_to_ref,
         )
         return ref
 
@@ -576,6 +596,7 @@ class Node(ABC):
         initial_params: Optional[Dict[str, Dict[str, Any]]] = None,
         conns: Optional[Tuple[Connection, Connection]] = None,
         init_ref: bool = True,
+        send_output_to_ref: bool = True,
     ) -> Tuple[NodeRef, "Node"]:
         """
         Create a new node instance in the current process and return a reference to the node,
@@ -585,13 +606,19 @@ class Node(ABC):
         `initial_params` : Optional[Dict[str, Dict[str, Any]]]
             A dictionary of parameter groups, where each group is a dictionary of parameter names and values.
             Defaults to None.
+        `conns` : Optional[Tuple[Connection, Connection]]
+            A tuple of two connections to use for the node. If None, a new connection pair will be created.
+        `init_ref` : bool
+            If True, call initialize() on the returned NodeRef. Defaults to True.
+        `send_output_to_ref` : bool
+            If True, connect the output slots of the node to the returned NodeRef. Defaults to True.
 
         ### Returns
         `Tuple[NodeRef, Node]`
             A tuple containing the node reference and the node itself.
         """
         # generate arguments for the node
-        in_slots, out_slots, params = cls._configure(cls)
+        in_slots, out_slots, params = cls._configure()
 
         # integrate initial parameters if they are provided
         if initial_params is not None:
@@ -616,6 +643,7 @@ class Node(ABC):
             deepcopy(params),
             cls,
             create_initialized=init_ref,
+            receive_node_outputs=send_output_to_ref,
         )
         return ref, node
 
@@ -629,7 +657,7 @@ class Node(ABC):
             The node instance.
         """
         # generate arguments for the node
-        in_slots, out_slots, params = cls._configure(cls)
+        in_slots, out_slots, params = cls._configure()
         # instantiate the node in the current process
         return cls(None, in_slots, out_slots, params, NodeEnv.STANDALONE)
 
@@ -659,7 +687,7 @@ class Node(ABC):
         return ""
 
     @property
-    def assets_path(self) -> PosixPath:
+    def assets_path(self) -> Path:
         """
         Returns the absolute path to the assets folder of goofi-pipe.
 
@@ -667,7 +695,7 @@ class Node(ABC):
         `PosixPath`
             The path to the assets folder of the node.
         """
-        return pkg_resources.files(assets)
+        return Path(str(pkg_resources.files(assets)))
 
     @property
     def data_path(self) -> str:
